<?php

namespace Intuit\Jobs;

use Carbon\Carbon;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldBeUnique;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Log;

use Intuit\Models\{Customer,ProviderToken};

class AccountingCustomerUpdate implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

	private const LOGKEY = 'JAS';

	private Customer $customer;
	private ProviderToken $to;
	private bool $forceprod;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct(ProviderToken $to,Customer $customer,bool $forcprod=FALSE)
    {
		$this->onQueue('intuit');

		$this->customer = $customer;
		$this->to = $to->withoutRelations();
		$this->forceprod = $forcprod;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
		Config::set('site',$this->to->site);

		$api = $this->to->provider->API($this->to,$this->forceprod);
		$updated = $api->updateCustomer(array_merge(
			$this->customer->getDirty(),
			[
				'Id'=>$this->customer->id,
				'SyncToken'=>$this->customer->synctoken,
			]));

		if (($x=$this->to->provider->accounts->where('pivot.ref',$updated->id))->count() === 1) {
			$ao = $x->pop();

			$ao->providers()->syncWithoutDetaching([
				$this->to->provider->id => [
					'ref' => $updated->id,
					'synctoken' => $updated->synctoken,
					'created_at'=>Carbon::create($updated->created_at),
					'updated_at'=>Carbon::create($updated->updated_at),
					'site_id'=>$ao->site_id,                         // @todo See if we can have this handled automatically
				],
			]);

			Log::info(sprintf('%s:Updated account [%s] (%s:%s), synctoken now [%s]',self::LOGKEY,$ao->sid,$updated->id,$updated->DisplayName,$updated->synctoken));

		} else {
			Log::error(sprintf('%s:Unable to update account refer for [%s:%s]',self::LOGKEY,$updated->id,$updated->DisplayName));
		}
    }
}
